/***********************************
 **  EMUTAG SDK MAIN HEADER FILE  **
 ***********************************/

#define F_CPU		13560000

/* LOCK SWITCH */
#define LOCK_PORT	PIND
#define LOCK_PIN	PD4

/* CONTROL & STATUS FLAGS used to pass conditions to and from user code */
register volatile uint8_t ctrl_flags asm("r16");
#define F_RX_CHR_ERR	0
// Bit 0 becomes set if collision or field loss is detected (2 consecutive gaps in carrier in 1 Manchester-encoded bit).
#define F_RX_OVF_ERR	1
// Bit 1 becomes set if the incoming message is longer than 180 bits (20 data bytes) to indicate overflow.
#define F_RX_PAR_ERR	2
// Bit 2 becomes set if any of the received characters contains a parity error.
#define F_TX_ENABLE	3
// Set bit 3 to transmit a reply packet. Clear bit 3 to disable transmit, reply() should still be called after!
#define F_TX_CRC	4
// Set bit 4 to automatically calculate response CRC and append it to the response. Clear bit 4 to send "as is".
// CRC is stored at the end of tx_buf[] before transmission, so maximum data payload with CRC is 18 bytes.
#define F_TX_PAR_OFF	5
// Set bit 5 to disable transmission of parity bits after EVERY message byte. Clear bit 5 to transmit parity bits.
#define F_ST_HALT	6
// Bit 6 is automatically cleared when a loss of carrier field is detected for >100 usec.
#define F__RESERVED	7
// Reserved for internal operations of main code. Do not write.
// Bits 0 - 5 are automatically cleared every time user_proc() exits, and after initialization.

/* STATE is a shared register that the user is free to read/write, but is reset to 0 on carrier loss for >100 usec */
register volatile uint8_t state asm("r6");
#define S_IDLE		0

/* RESERVED REGISTERS, DO NOT WRITE */
register volatile uint8_t _reserved_0 asm("r2");
register volatile uint8_t _reserved_1 asm("r3");
register volatile uint8_t _reserved_2 asm("r4");
register volatile uint8_t _reserved_3 asm("r5");
/* ALSO, DO NOT WRITE r0, r1, r18-r31, as they are often used by avr-gcc */

/* RECEIVE & TRANSMIT BUFFERS: read & write them to communicate, size is fixed to 20 bytes */
#define RX_BUF_SIZE	20
#define TX_BUF_SIZE	20
extern uint8_t rx_buf[RX_BUF_SIZE];
extern uint8_t tx_buf[TX_BUF_SIZE];

/* FUNCTIONS */

/*  crc_chk() : Used to check CRC of received message.
 *
 *  end = length of message in rx_buf[] including CRC
 *  returns 0 if CRC is correct, 1 otherwise
 */
extern uint8_t crc_chk(uint8_t end);

/*  reply() : Used to terminate user function and return control to main.
 *          : If nothing needs to be transmitted, call with any parameters with F_TX_ENABLE = 0.
 *
 *  start   = offset of first byte to transmit from tx_buf[]
 *  tx_bits = number of bits to transmit from the first byte by left-shifting it out of tx_buf[], if < 8
 *  tx_end  = offset of last byte to transmit from tx_buf[]
 */
extern void reply(uint8_t tx_start, uint8_t tx_bits, uint8_t tx_end);

